@@ -21,6 +21,7 @@ install("tweepy")
 install("nltk")
 install("textblob")
 install("fastdtw")
+install ("spicy")
 ##----------------------------------------------IMPORTS---------------------------------------------------------------------------#	
 import json
 import itertools
@@ -38,6 +39,7 @@ import requests
 import matplotlib.pyplot as plt
 import tweepy
 from os import path
+import os
 from datetime import timedelta
 from datetime import datetime
 
@@ -93,7 +95,7 @@ def profile_connections (data_tweets):
     lst_col = "screen_name_mention"
     data = data[["screen_name","screen_name_mention"]]
 	#Create new rows in case of a tweet has different mentions
-    data_ext =pd.DataFrame({col:np.repeat(data[col].values, data[lst_col].str.len())for col in data.columns.difference(
+    data_ext = pd.DataFrame({col:np.repeat(data[col].values, data[lst_col].str.len())for col in data.columns.difference(
         [lst_col])}).assign(**{lst_col:np.concatenate(data[lst_col].values)})[data.columns.tolist()]
     data1_ext = pd.DataFrame({'iteration' : data_ext.groupby(data_ext.columns.tolist(),as_index=False).size().sort_values(
                                                                         ascending = False)}).reset_index()
@@ -295,17 +297,17 @@ def user_timeline_extraction(node_list, consumerKey, consumerSecret,accessToken,
                               "favourites_count": "favorites_count",
                               "full_text": "text"})
         df2 = metadata_extraction(df)
-        if path.exists(filepath + filename):
-          df2.to_csv(filename,mode="a", index=False, header=False)
+        if path.exists(os.path.join(filepath,filename)):
+          df2.to_csv(path_or_buf =os.path.join(filepath,filename),mode="a", index=False, header=False)
         else:
-          df2.to_csv(filename,index=False)         
+          df2.to_csv(path_of_buf = os.path.join(filepath,filename),index=False)         
         if len(list_users_noInfo) !=0:
           nodes = pd.DataFrame()
           nodes["nodos"] = list_users_noInfo
-          if path.exists(filepath+"node_without_"+filename):
-            nodes.to_csv("node_without_"+ filename, mode ="a",index=False, header=False)          
+          if path.exists(os.path.join(filepath,"node_without_",filename)):
+            nodes.to_csv(os.path.join(filepath,"node_without_",filename), mode ="a",index=False, header=False)          
           else:
-             nodes.to_csv("node_without_"+ filename, index=False)
+             nodes.to_csv(os.path.join(filepath,"node_without_",filename), index=False)
         k = k+profile_iter        
         if ultima ==1:          
           ultima =2
@@ -628,47 +630,45 @@ def time_series_tw (df):
     return df 
 
 #-------------------------------------------Ratios and other advanced features----------------------------------#
-def duplicates_detection(data):
-    data["possible_duplicates"] = 0
-    for i in range(len(data)):
-        if i < len(data):
-            possible_duplicated = []
-            for j in range(len(data)):
-                if j !=i:
-                    list_dtw_1 = np.array(data["tw_per_day_list"].iloc[i])              
-                    list_dtw_2 = np.array(data["tw_per_day_list"].iloc[j])
-                    distance, path = fastdtw(list_dtw_1,list_dtw_2)
-                    if (distance < 600):
-                        possible_duplicated.append({"screen_name":data["screen_name"].iloc[j], "distance": distance})
-            if (len(possible_duplicated)==0):
-                possible_duplicated.append({"screen_name": "Empty", "distance": "Empty", "tw_similarity": 0,"tw_match_tw": "Empty"})   
-                data["possible_duplicates"].iloc[i] = possible_duplicated        
-            else: 
-                data["possible_duplicates"].iloc[i] = possible_duplicated          
-                for t in range(len(data["possible_duplicates"].iloc[i])):
-                    #text_1 is the differences possible duplicates with the profile evaluate
-                    text_1_aux = pd.DataFrame({"text":data[data["screen_name"] == data["possible_duplicates"].iloc[i][t]["screen_name"]]["text_tweets_days_list"]})
-                    text_1 = pd.DataFrame()
-                    for a in range(len(text_1_aux)):
-                        aux2 = pd.DataFrame({'text':text_1_aux["text"].iloc[a]})
-                        text_1 = text_1.append(aux2, ignore_index=False).reset_index(drop=True)
-                    text_1 = list(itertools.chain(*text_1["text"]))   
-                    text_2 = list(itertools.chain(*data.iloc[i]["text_tweets_days_list"]))               
-                    if "Empty" in text_1:
-                      text_1 = list(filter(("Empty").__ne__, text_1))
-                    if "Empty" in text_2:  
-                      text_2 = list(filter(("Empty").__ne__, text_2))
-                    if "" in text_1:
-                      text_1 = list(filter(("").__ne__, text_1))
-                    if "" in text_2:  
-                      text_2 = list(filter(("").__ne__, text_2))  
-                    comparacion = [item for item in text_2 if item in text_1]
-                    if len(comparacion) > 0:
-                        data["possible_duplicates"].iloc[i][t]["tw_similarity"] = len(comparacion)/(data["num_userTw"].iloc[i])
-                        data["possible_duplicates"].iloc[i][t]["text_match_tw"] = comparacion
-                    else:
-                        data["possible_duplicates"].iloc[i][t]["tw_similarity"] = 0
-                        data["possible_duplicates"].iloc[i][t]["text_match_tw"] = "Empty"
+def duplicates_detection(data, i):
+    if i < len(data):
+        possible_duplicated = []
+        for j in range(len(data)):
+            if j !=i:
+                list_dtw_1 = np.array(data["tw_per_day_list"].iloc[i])              
+                list_dtw_2 = np.array(data["tw_per_day_list"].iloc[j])
+                distance, path = fastdtw(list_dtw_1,list_dtw_2)
+                if (distance < 600):
+                    possible_duplicated.append({"screen_name":data["screen_name"].iloc[j], "distance": distance})
+        if (len(possible_duplicated)==0):
+            possible_duplicated.append({"screen_name": "Empty", "distance": "Empty", "tw_similarity": 0,"tw_match_tw": "Empty"})   
+            data["possible_duplicates"].iloc[i] = possible_duplicated        
+        else: 
+            data["possible_duplicates"].iloc[i] = possible_duplicated          
+            for t in range(len(data["possible_duplicates"].iloc[i])):
+                #text_1 is the differences possible duplicates with the profile evaluate
+                text_1_aux = pd.DataFrame({"text":data[data["screen_name"] == data["possible_duplicates"].iloc[i][t]["screen_name"]]["text_tweets_days_list"]})
+                text_1 = pd.DataFrame()
+                for a in range(len(text_1_aux)):
+                    aux2 = pd.DataFrame({'text':text_1_aux["text"].iloc[a]})
+                    text_1 = text_1.append(aux2, ignore_index=False).reset_index(drop=True)
+                text_1 = list(itertools.chain(*text_1["text"]))   
+                text_2 = list(itertools.chain(*data.iloc[i]["text_tweets_days_list"]))               
+                if "Empty" in text_1:
+                  text_1 = list(filter(("Empty").__ne__, text_1))
+                if "Empty" in text_2:  
+                  text_2 = list(filter(("Empty").__ne__, text_2))
+                if "" in text_1:
+                  text_1 = list(filter(("").__ne__, text_1))
+                if "" in text_2:  
+                  text_2 = list(filter(("").__ne__, text_2))  
+                comparacion = [item for item in text_2 if item in text_1]
+                if len(comparacion) > 0:
+                    data["possible_duplicates"].iloc[i][t]["tw_similarity"] = len(comparacion)/(data["num_userTw"].iloc[i])
+                    data["possible_duplicates"].iloc[i][t]["text_match_tw"] = comparacion
+                else:
+                    data["possible_duplicates"].iloc[i][t]["tw_similarity"] = 0
+                    data["possible_duplicates"].iloc[i][t]["text_match_tw"] = "Empty"
     return data
 	
 	
@@ -720,12 +720,12 @@ def advanced_features(data):
         else:
             data["max_ret_tw"].iloc[i] = max((data["retweet_tweets_list"].iloc[i]))
           
-        #data = duplicates_detection(data, i)
+        data = duplicates_detection(data, i)
         data["max_tw_day"].iloc[i] =max((data["tw_per_day_list"].iloc[i]))
         data["days_with_tw"].iloc[i] = len(data["tw_per_day_list"].iloc[i])
         data["index_max_day_tw"].iloc[i] = data["tw_per_day_list"].iloc[i].index(data["max_tw_day"].iloc[i])        
     data = trend_tweets_features(data) 
-    #data = close_duplicates(data)
+    data = close_duplicates(data)
         
     return data
 	
@@ -820,63 +820,137 @@ def text_to_list_features_conversor (data):
 	
 
 #----------------------------------------------Filters-------------------------------------------------------#
-
-
-
 def abnormal_categories_profiles(data_join):
-    data_join["evaluate_NLP"] ="No"   
-    
-    
-    #1.Profiles old spreader
-    data_join["evaluate_NLP"][(data_join["index_max_day_tw"] >=30) & (data_join["profile_type"]==False) &
-                  (data_join["max_tw_day"]>60)] ="NLP-F0"  
-    
-    #2.Profiles with low number of tweets but with a high iteration
-    data_join["evaluate_NLP"][(data_join["trend"] == 0) & (data_join["profile_type"]==False) &
-                  (data_join["max_tw_day"]<40) & (data_join["tw_RT_rate"]<0.2) &
-                 ((data_join["max_ret_tw"]>500) | (data_join["max_fav_tw"]>500))] ="NLP-F1"
-    
-    #3.Profiles with a high number of RT in a specific day (try to spread information)
-    data_join["evaluate_NLP"][(data_join["trend"] == 0) & (data_join["profile_type"]==False) &
-                  (data_join["max_tw_day"]>150) & (data_join["tw_RT_rate"]<0.1) & 
-                (data_join["evaluate_NLP"]=="No")] = "NLP-F2"
-    
-    #4.Possible influencer profile
-    data_join["evaluate_NLP"][(data_join["trend"]==1) & (data_join["profile_type"]==False) &
-             (data_join["in_degree"]>=5) & (data_join["followers"]>=3000) &
-             (data_join["evaluate_NLP"]=="No")] = "NLP-F3"
-    
-    #5.Possible constant spreader
-    data_join["evaluate_NLP"][(data_join["trend"]==1) & (data_join["profile_type"]==False) &
-             (data_join["out_degree"]>=2) & (data_join["following"]>=3000) & (data_join["evaluate_NLP"]=="No")] = "NLP-F4"                
 
-    #6.Possible new account with a high activity last days
-    data_join["evaluate_NLP"][(data_join["trend"]==2) & (data_join["profile_type"]==False) &
-             (data_join["twitter_years"]==1) & (data_join["max_tw_day"]>210) &
-              (data_join["evaluate_NLP"]=="No")] = "NLP-F5"                     
-
-    #7.Possible fake behaviour profile in the last days
-    data_join["evaluate_NLP"][(data_join["trend"]==2) & (data_join["profile_type"]==False) &
-             (data_join["fake_sum"]>=2) & (data_join["evaluate_NLP"]=="No")] = "NLP-F6" 
     
-    #8.Possible influencers profile in the last days with a high iteration
-    data_join["evaluate_NLP"][(data_join["trend"]==2) & (data_join["profile_type"]==False) &
-             ((data_join["max_ret_tw"]>500) | (data_join["max_fav_tw"]>500)) &
-                  (data_join["max_tw_day"]>210) &  (data_join["evaluate_NLP"]=="No")] = "NLP-F7"
-    
-    #9.Profiles BOT (more than tweet or RT each 5 minutes the 24 hours)
-    data_join["evaluate_NLP"][(data_join["max_tw_day"]>	288) &(data_join["profile_type"]==False) &
-                             (data_join["evaluate_NLP"]=="No")] ="NLP-8"
-							 
-    #10. More data required
-    data_join["evaluate_NLP"][(data_join["trend"]==2) & (data_join["profile_type"]==False) & 
-                              (data_join["max_tw_day"]>180) & (data_join["evaluate_NLP"]=="No")] = "More Data"
+    data_join["evaluate_NLP"] = "0"
+    for i in range(len(data_join)):
+        abnormal_category_belonging = []
+        #1.Profiles old spreader
+        if ((data_join["index_max_day_tw"].iloc[i] >=30) & (data_join["profile_type"].iloc[i]==False) &
+                      (data_join["max_tw_day"].iloc[i]>60)):
+            abnormal_category_belonging.append("Old Spreader")  
+        
+        #2.Profiles with low number of tweets but with a high iteration
+        if((data_join["trend"].iloc[i] == 0) & (data_join["profile_type"].iloc[i]==False) &
+                      (data_join["max_tw_day"].iloc[i]<40) & (data_join["tw_RT_rate"].iloc[i]<0.2) &
+                     ((data_join["max_ret_tw"].iloc[i]>500) | (data_join["max_fav_tw"].iloc[i]>500))):
+            abnormal_category_belonging.append("LRT_HI")
+            
+        #3.Profiles with a high number of RT in a specific day (try to spread information)
+        if((data_join["trend"].iloc[i] == 0) & (data_join["profile_type"].iloc[i]==False) &
+                      (data_join["max_tw_day"].iloc[i]>150) & (data_join["tw_RT_rate"].iloc[i]<0.1)):
+            abnormal_category_belonging.append("HRT_SD")
+            
+        #4.Possible influencer profile
+        if((data_join["trend"].iloc[i]==1) & (data_join["profile_type"].iloc[i]==False) &
+                 (data_join["in_degree"].iloc[i]>=5) & (data_join["followers"].iloc[i]>=3000)):
+            abnormal_category_belonging.append("Influencer")
+        
+        #5.Possible constant spreader
+        if((data_join["trend"].iloc[i]==1) & (data_join["profile_type"].iloc[i]==False) &
+                 (data_join["out_degree"].iloc[i]>=2) & (data_join["following"].iloc[i]>=3000)):
+            abnormal_category_belonging.append("Constant Spreader")                
 
+        #6.Possible new account with a high activity last days
+        if((data_join["trend"].iloc[i]==2) & (data_join["profile_type"].iloc[i]==False) &
+                 (data_join["twitter_years"].iloc[i]==1) & (data_join["max_tw_day"].iloc[i]>210)):
+            abnormal_category_belonging.append("NP_HA")                     
+
+        #7.Possible fake behaviour profile in the last days
+        if((data_join["trend"].iloc[i]==2) & (data_join["profile_type"].iloc[i]==False) &
+                 (data_join["fake_sum"].iloc[i]>=2)):
+            abnormal_category_belonging.append("Fake Behaviour") 
+        
+        #8.Possible influencers profile in the last days with a high iteration
+        if((data_join["trend"].iloc[i]==2) & (data_join["profile_type"].iloc[i]==False) &
+                 ((data_join["max_ret_tw"].iloc[i]>500) | (data_join["max_fav_tw"].iloc[i]>500)) &
+                      (data_join["max_tw_day"].iloc[i]>210)):
+            abnormal_category_belonging.append("Influ_LD_HI")
+        
+        #9.Profiles BOT (more than tweet or RT each 5 minutes the 24 hours)
+        if((data_join["max_tw_day"].iloc[i]>288) &(data_join["profile_type"].iloc[i]==False)):
+            abnormal_category_belonging.append("BOT")
+                                 
+        #10. More data required
+        if((data_join["trend"].iloc[i]==2) & (data_join["profile_type"].iloc[i]==False) & 
+                                  (data_join["max_tw_day"].iloc[i]>180)):
+            abnormal_category_belonging.append("More Data")
+        
+        if len(abnormal_category_belonging)!=0:
+            data_join["evaluate_NLP"].iloc[i] = abnormal_category_belonging     
+        else:
+            abnormal_category_belonging.append("No")
+            data_join["evaluate_NLP"].iloc[i] = abnormal_category_belonging     
+        
+    return data_join
+
+
+def abnormal_categories_profiles2(data_join):
+
+    
+    data_join["evaluate_NLP"] = "No"
+    for i in range(len(data_join)):
+        abnormal_category_belonging = []
+        #1.Profiles old spreader
+        if ((data_join["index_max_day_tw"].iloc[i] >=30) & (data_join["profile_type"].iloc[i]==False) &
+                      (data_join["max_tw_day"].iloc[i]>60)):
+            abnormal_category_belonging.append("Old Spreader")  
+        
+        #2.Profiles with low number of tweets but with a high iteration
+        if((data_join["trend"].iloc[i] == 0) & (data_join["profile_type"].iloc[i]==False) &
+                      (data_join["max_tw_day"].iloc[i]<40) & (data_join["tw_RT_rate"].iloc[i]<0.2) &
+                     ((data_join["max_ret_tw"].iloc[i]>500) | (data_join["max_fav_tw"].iloc[i]>500))):
+            abnormal_category_belonging.append("LRT_HI")
+            
+        #3.Profiles with a high number of RT in a specific day (try to spread information)
+        if((data_join["trend"].iloc[i] == 0) & (data_join["profile_type"].iloc[i]==False) &
+                      (data_join["max_tw_day"].iloc[i]>150) & (data_join["tw_RT_rate"].iloc[i]<0.1)):
+            abnormal_category_belonging.append("HRT_SD")
+            
+        #4.Possible influencer profile
+        if((data_join["trend"].iloc[i]==1) & (data_join["profile_type"].iloc[i]==False) &
+                 (data_join["in_degree"].iloc[i]>=5) & (data_join["followers"].iloc[i]>=3000)):
+            abnormal_category_belonging.append("Influencer")
+        
+        #5.Possible constant spreader
+        if((data_join["trend"].iloc[i]==1) & (data_join["profile_type"].iloc[i]==False) &
+                 (data_join["out_degree"].iloc[i]>=2) & (data_join["following"].iloc[i]>=3000)):
+            abnormal_category_belonging.append("Constant Spreader")                
+
+        #6.Possible new account with a high activity last days
+        if((data_join["trend"].iloc[i]==2) & (data_join["profile_type"].iloc[i]==False) &
+                 (data_join["twitter_years"].iloc[i]==1) & (data_join["max_tw_day"].iloc[i]>210)):
+            abnormal_category_belonging.append("NP_HA")                     
+
+        #7.Possible fake behaviour profile in the last days
+        if((data_join["trend"].iloc[i]==2) & (data_join["profile_type"].iloc[i]==False) &
+                 (data_join["fake_sum"].iloc[i]>=2)):
+            abnormal_category_belonging.append("Fake Behaviour") 
+        
+        #8.Possible influencers profile in the last days with a high iteration
+        if((data_join["trend"].iloc[i]==2) & (data_join["profile_type"].iloc[i]==False) &
+                 ((data_join["max_ret_tw"].iloc[i]>500) | (data_join["max_fav_tw"].iloc[i]>500)) &
+                      (data_join["max_tw_day"].iloc[i]>210)):
+            abnormal_category_belonging.append("Influ_LD_HI")
+        
+        #9.Profiles BOT (more than tweet or RT each 5 minutes the 24 hours)
+        if((data_join["max_tw_day"].iloc[i]>288) &(data_join["profile_type"].iloc[i]==False)):
+            abnormal_category_belonging.append("BOT")
+                                 
+        #10. More data required
+        if((data_join["trend"].iloc[i]==2) & (data_join["profile_type"].iloc[i]==False) & 
+                                  (data_join["max_tw_day"].iloc[i]>180)):
+            abnormal_category_belonging.append("More Data")
+        
+        if len(abnormal_category_belonging)!=0:
+            data_join["evaluate_NLP"].iloc[i] = abnormal_category_belonging          
+        
     return data_join
 
 
 	
-def more_tweets_required(df,df_graph_features,consumerKey,consumerSecret,accessToken,accessTokenSecret,filepath,filename,n_groups,save_by):
+def more_tweets_required2(df,df_graph_features,consumerKey,consumerSecret,accessToken,accessTokenSecret,filepath,filename,n_groups,save_by):
 	df_evaluate = df[(df["evaluate_NLP"] !="No") & (df["evaluate_NLP"] !="More Data")]
 	node_list_moreData = list(df[df["evaluate_NLP"]=="More Data"]["screen_name"])
 	if len(node_list_moreData) >0:
@@ -890,7 +964,35 @@ def more_tweets_required(df,df_graph_features,consumerKey,consumerSecret,accessT
 		df_evaluate = df_evaluate.append(df_metadata_moreData, ignore_index=True)
 	return df_evaluate
 	
-
+def more_tweets_required(df,df_graph_features,consumerKey,consumerSecret,accessToken,accessTokenSecret,filepath,filename,n_groups,save_by):
+    print(len(df))
+    df_evaluate = df.copy()
+    node_list_moreData = []
+    delete_rows = []
+    for i in range(len(df)):
+        if "More Data" in df["evaluate_NLP"].iloc[i]:
+            node_list_moreData.append(df["screen_name"].iloc[i]) 
+            delete_rows.append(i)           
+        if "No" in df["evaluate_NLP"].iloc[i]:
+            delete_rows.append(i)
+    df_evaluate= df_evaluate.drop(df_evaluate.index[delete_rows]).reset_index(drop=True)
+    print(len(df_evaluate))
+    if len(node_list_moreData) >0:
+        df_metadata_moreData, list_more_Data_noData = user_timeline_extraction(node_list_moreData,consumerKey,consumerSecret,accessToken,accessTokenSecret,filepath, filename,n_groups,save_by)
+        df_metadata_moreData = pd.read_csv(filepath + filename)
+        df_metadata_moreData = text_to_list_features_conversor(df_metadata_moreData)
+        df_metadata_moreData = advanced_features(df_metadata_moreData)
+        df_metadata_moreData = merge_dataframes(df_metadata_moreData,df_graph_features)
+        df_metadata_moreData = abnormal_categories_profiles(df_metadata_moreData)
+        print(len(df_metadata_moreData))
+        delete_rows = []
+        for i in range(len(df_metadata_moreData)):
+          if (("More Data" in df_metadata_moreData["evaluate_NLP"].iloc[i]) | ("No" in df_metadata_moreData["evaluate_NLP"].iloc[i])): 
+              delete_rows.append(i)
+        df_metadata_moreData= df_metadata_moreData.drop(df.index[delete_rows])
+        df_metadata_moreData= df_metadata_moreData.reset_index(drop=True)
+        df_evaluate = df_evaluate.append(df_metadata_moreData, ignore_index=True)
+    return df_evaluate
 
 def contextual_analysis_profile(data, keyword_corpus, hashtag_corpus):  
     #To remember which name of t he tw_per_day_list belongs each tweets
@@ -906,7 +1008,7 @@ def contextual_analysis_profile(data, keyword_corpus, hashtag_corpus):
     tweets_df = keyword_score(tweets_df, sample_data_tokenized,keyword_corpus)
     tweets_df = hashtag_score(tweets_df,hashtag_corpus)	
     tweets_df["terrorist_suggestion"] = 0
-    tweets_df["terrorist_suggestion"][(tweets_df["hashtag_score"]>0) | (tweets_df["keyword_score"]>0)] = 1
+    tweets_df["terrorist_suggestion"][(tweets_df["hashtag_score"]>0) | (tweets_df["keyword_score"]>0.3)] = 1
     tweets_irr_words = len(tweets_df[tweets_df["terrorist_suggestion"]==1])
     if len(tweets_df[tweets_df["terrorist_suggestion"]==1]) == 0:
       data["per_hash_key"] = 0
@@ -1130,7 +1232,7 @@ def keyword_score(data,sample_data_tokenized, keyword_corpus):
 
 
 def terrorist_keywords_belonging(data):
-	temp = data[data['keyword_score']>0].reset_index(drop=True)
+	temp = data[data['keyword_score']>0.3].reset_index(drop=True)
 	# Draw plot
 	fig, ax = plt.subplots(figsize=(36,8), dpi= 80)
 	ax.vlines(x=temp.index, ymin=0, ymax=temp.keyword_score, color='firebrick', alpha=0.7, linewidth=2)
@@ -1149,7 +1251,7 @@ def terrorist_keywords_belonging(data):
 	
 def pie_chart_keywords_terrorist(data):
 	plt.rcParams['figure.figsize'] = [9, 6]
-	data_terr = data[data['keyword_score']>0.3].reset_index(drop=True)
+	data_terr = data[data['keyword_score']>0.2].reset_index(drop=True)
 	p=0
 	n=0
 	o=0
